/**
 * DevExtreme (ui/scheduler/workspaces/ui.scheduler.virtual_scrolling.js)
 * Version: 20.2.1 (build 20232-1710)
 * Build date: Wed Aug 19 2020
 *
 * Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.default = void 0;

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}
var ROW_HEIGHT = 50;
var VirtualScrolling = function() {
    function VirtualScrolling(workspace, viewportHeight, scrollable) {
        _classCallCheck(this, VirtualScrolling);
        this._workspace = workspace;
        this._viewportHeight = viewportHeight;
        this._scrollable = scrollable;
        this._init()
    }
    _createClass(VirtualScrolling, [{
        key: "getWorkspace",
        value: function() {
            return this._workspace
        }
    }, {
        key: "getViewportHeight",
        value: function() {
            return this._viewportHeight
        }
    }, {
        key: "getScrollable",
        value: function() {
            return this._scrollable
        }
    }, {
        key: "getRowHeight",
        value: function() {
            return ROW_HEIGHT
        }
    }, {
        key: "getState",
        value: function() {
            return this._state
        }
    }, {
        key: "_init",
        value: function() {
            var pageSize = Math.ceil(this.getViewportHeight() / this.getRowHeight());
            var scrollOffset = {
                top: 0,
                left: 0
            };
            this._state = {
                pageSize: pageSize,
                scrollOffset: scrollOffset,
                startIndex: -1,
                rowCount: 0,
                topVirtualRowCount: 0,
                bottomVirtualRowCount: 0,
                topVirtualRowHeight: 0,
                bottomVirtualRowHeight: 0
            };
            this._updateState(scrollOffset);
            this._attachScrollableEvent()
        }
    }, {
        key: "_attachScrollableEvent",
        value: function() {
            var _this = this;
            var scrollable = this.getScrollable();
            var onScroll = scrollable.option("onScroll");
            scrollable.option("onScroll", function(e) {
                if (onScroll) {
                    onScroll.apply(scrollable, [e])
                }
                var scrollOffset = null === e || void 0 === e ? void 0 : e.scrollOffset;
                if (scrollOffset && _this._updateState(scrollOffset)) {
                    _this.getWorkspace().renderRWorkspace()
                }
            })
        }
    }, {
        key: "_updateState",
        value: function(scrollOffset) {
            var workspace = this.getWorkspace();
            var state = this.getState();
            var top = scrollOffset.top;
            var currentStartIndex = state.startIndex;
            var rowHeight = this.getRowHeight();
            state.scrollOffset = scrollOffset;
            state.startIndex = Math.floor(top / rowHeight);
            if (currentStartIndex !== state.startIndex) {
                var pageSize = state.pageSize;
                var groupCount = workspace._getGroupCount();
                var topVirtualRowCount = Math.floor(top / rowHeight);
                var totalRowCount = workspace._getTotalRowCount(groupCount);
                var deltaRowCount = totalRowCount - topVirtualRowCount;
                var rowCount = deltaRowCount >= pageSize ? pageSize : deltaRowCount;
                var bottomVirtualRowCount = totalRowCount - topVirtualRowCount - rowCount;
                if (workspace.isGroupedAllDayPanel()) {
                    bottomVirtualRowCount -= 1
                }
                state.topVirtualRowCount = topVirtualRowCount;
                state.rowCount = rowCount;
                state.bottomVirtualRowCount = bottomVirtualRowCount;
                this._updateStateCore();
                return true
            }
            return false
        }
    }, {
        key: "_updateStateCore",
        value: function() {
            var state = this.getState();
            var rowHeight = this.getRowHeight();
            var topVirtualRowCount = state.topVirtualRowCount;
            var bottomVirtualRowCount = state.bottomVirtualRowCount;
            var prevTopVirtualRowHeight = state.topVirtualRowHeight;
            var prevBottomVirtualRowHeight = state.bottomVirtualRowHeight;
            var topVirtualRowHeight = rowHeight * topVirtualRowCount;
            var bottomVirtualRowHeight = rowHeight * bottomVirtualRowCount;
            var isAppend = prevTopVirtualRowHeight < topVirtualRowHeight;
            var isPrepend = prevBottomVirtualRowHeight < bottomVirtualRowHeight;
            var needAddRows = isAppend || isPrepend;
            if (needAddRows) {
                state.topVirtualRowHeight = topVirtualRowHeight;
                state.bottomVirtualRowHeight = bottomVirtualRowHeight
            }
        }
    }, {
        key: "dispose",
        value: function() {}
    }]);
    return VirtualScrolling
}();
exports.default = VirtualScrolling;
module.exports = exports.default;
